import { Tr } from 'pi_ucore/db/mgr';
import { getCurrentRid } from "pi_common_tools/main";
import { LotteryDrop, Lottery } from "game_roulette_server/xls/lottery_dial/lottery.struct";
import { rouletteDb } from "game_roulette_server/struct/roulette_db.struct";
import { read, write } from 'pi_common_tools/util/db_tool';
import { getAllCfg, getCfg } from "pi_common_tools/util/db_tool";
import { sendAwards } from "pi_award/api";
import { getMultiAwards } from "pi_award/util";
import { getInitialVByKey } from "game_activity_initial_value/api";
import { getTotalV } from "game_recharge_server/api";
import { randomIndexByWeight } from "pi_common_tools/util/random";
// 红点
// #[rpc=rpcServer,src=*,rpcWrapper=read,rpcMode=check]
export const checkRedPoint = async (tr: Tr, rid: number, rouletteID: number) => {
    let count = await getCount(tr, rid, rouletteID);
    let dbInfo = await read(tr, rouletteDb, `${rid}_${rouletteID}`);
    return dbInfo.count < count ? 1 : 0;
}

// #[rpc=rpcServer,src=*,rpcWrapper=read,rpcMode=check]
export const getDrawCount = async (tr: Tr, rid: number, rouletteID: number) => {
    let count = await getCount(tr, rid, rouletteID);
    let dbInfo = await read(tr, rouletteDb, `${rid}_${rouletteID}`);
    return count - dbInfo.count > 0 ? count - dbInfo.count : 0;
}
// #[rpc=rpcServer,src=*,rpcWrapper=read,rpcMode=check]
export const getDropCfg = async (tr: Tr, rid: number) => {
    return getAllCfg(LotteryDrop);
}

// 抽奖
// #[rpc=rpcServer, src=*,rpcMode=remote, rpcWrapper=write]
export const rouletteLottery = async (tr: Tr, rid: number, rouletteID: number) => {
    let dbInfo = await read(tr, rouletteDb, `${rid}_${rouletteID}`);
    let count = await getCount(tr, rid, rouletteID);
    if (dbInfo.count >= count) {
        return "没有抽取次数";
    }
    let propList = [], wightList = [], itemInfo = [];
    let confAll = getAllCfg(LotteryDrop);
    for (let elConf of confAll) {
        let conf = elConf[1] as LotteryDrop;
        if (conf.group == rouletteID && conf.lottery_pro !== 0) {
            propList.push(conf.id);
            wightList.push(conf.lottery_pro);
            itemInfo.push(conf.lottery_dis);
        }
    }
    let showList = [];
    let id = randomIndexByWeight(propList, wightList);

    let awardConf = <LotteryDrop>await getCfg(LotteryDrop, id);
    let awardList = getMultiAwards(awardConf.award, awardConf.lottery_dis);

    showList.push(id);
    dbInfo.count += 1;
    if (awardList.length > 0) {
        let success = await sendAwards(tr, rid, awardList);
        if (!success) {
            return "奖励发送失败";
        }
    }
    await write(tr, rouletteDb, dbInfo.key, dbInfo);
    return { showList, awardList };
}


const getCount = async (tr: Tr, rid: number, rouletteID: number) => {
    let payV = await getTotalV(tr, rid);
    let key = `${rid}_${rouletteID}`;
    let initV = await getInitialVByKey(tr, rid, key);
    let count = Math.floor((payV - initV) / 100);
    //根据配置表迭代获取次数信息
    let confMap = getAllCfg(Lottery);
    let totalCount = 0;
    for (let elConf of confMap) {
        let conf = elConf[1] as Lottery;
        if (conf.group === rouletteID && count >= conf.cost_money) {
            totalCount += conf.lottery_num
        } else {
            break;
        }
    }
    return totalCount;
}